#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sysexits.h>
#include <errno.h>
#include <ctype.h>

#include "parsecfg.h"

extern struct options config_options[];

int parse_config_file(char *config_file)
{
  FILE *config_fileptr = NULL;
  char buffer[MAXSTRING + 1];
  int line_number = 0;
  int status = EX_OK;

  config_fileptr = fopen(config_file, "r");

  if ( NULL == config_fileptr )
  {
    fprintf(stderr, "Error: Cannot read from config file '%s': %s\n", 
            config_file, strerror(errno));

    status = EX_NOINPUT;
  }
  else
  {
    /* Scroll through our config file looking for config info */
    while( NULL != fgets(buffer, sizeof(buffer), config_fileptr) )
    {
      line_number++;

      /* It's a commented or empty line, skip it */
      if ( *buffer == '\n' || *buffer == '\0' || *buffer == '#' )
      {
        continue;
      }

      /* Now we process the config info in the line */
      if ( EX_OK != parse_config_line(buffer) )
      {
        fprintf(stderr, "Error: problem with config file %s: line %d: %s", 
                config_file, line_number, buffer);
  
        status = EX_CONFIG;
  
        break;
      }
    }
  }

  return(status);
}

int parse_config_line(char *config_line)
{
  int status = EX_OK;
  char *str_copy = NULL;
  char *key = NULL;
  char *value = NULL;
  char *placeholder = NULL;
  int i = 0;

  str_copy = strdup(config_line);

  if ( NULL == str_copy )
  {
    fprintf(stderr, "\nError: cannot allocate memory\n\n");

    status = EX_OSERR;
  }
  else
  {
    /* config line syntax is "key = value" */
    key = strtok_r(str_copy, "=", &placeholder);

    /* Remove trailing white space from our key string */
    key = rtrim(key);

    value = strtok_r(NULL, "", &placeholder);

    /* Remove leading white space from our value string */
    value = ltrim(value);

    /* get rid of the newline at the end of our value string */
    value[strlen(value) - 1] = '\0';

    /* make our key all lowercase, for easier processing */
    key = strlower(key);

    int match = 0;

    while( NULL != (void *)config_options[i].optionkey )
    {
      if ( ( 0 == strncmp(key, config_options[i].optionkey, 
                          strlen(config_options[i].optionkey)) ) && 
           ( strlen(key) == strlen(config_options[i].optionkey) )
         )
      {
        match = 1;

        break;
      }

      i++;
    }

    if ( match )
    {
      status = config_options[i].functionptr(value);
    }
    else
    {
      status = EX_CONFIG;
    }
  }

  return(status);
}

/* Trim trailing whitespace (from the right side of a string) */
char *rtrim(char *string)
{
  while ( isspace( *(string + strlen(string) - 1) ) )
  {
    *(string + strlen(string) - 1) = '\0';
  }

  return(string);
}

/* Trim leading whitespace (from the left side of a string) */
char *ltrim(char *string)
{
  while ( isspace( *string) )
  {
    memmove(string, string + 1, strlen(string));
  }

  return(string);
}

/* Trim all whitespace from within a string */
char *strip_space(char *string)
{
  char *cptr = string;

  while ( *cptr )
  {
    if ( isspace( *cptr ) )
    {
      memmove(cptr, cptr + 1, strlen(cptr));
    }

    if ( '\0' != *cptr )
    {
      cptr++;
    }
  }

  return(string);
}

/* Convert our string to all lowercase */
char *strlower(char *string)
{
  char *cptr = string;

  while ( *cptr )
  {
    *cptr = tolower((int)(*cptr));

    cptr++;
  }

  return(string);
}

